﻿using UnityEngine;
using System;

public class CaveCameraRig : MonoBehaviour {
    public enum CaveStereoOption { Auto, Mono, Left, Right, Stereo }
    public enum CaveScreenOption { Auto, Front, Floor, FrontAndFloor }
    public enum CaveCamera { FrontLeft, FloorLeft, FrontRight, FloorRight }

    public const int CAVE_LAYER = 31;
	public bool debug = false;
	[ReadOnlyWhenPlaying] public bool renderToTexture = false;
	[ReadOnlyWhenPlaying] public Vector2Int renderTextureSize = new Vector2Int(2560, 1600);

    // select which views to render (in side-by-side stereo, front at top, floor at bottom)
    public CaveStereoOption stereoMode = CaveStereoOption.Stereo;
    public CaveScreenOption screens = CaveScreenOption.FrontAndFloor;
    // save current state to compare with new settings in each frame
    protected CaveStereoOption _stereoMode;
    protected CaveScreenOption _screens;

    public float interOccularDistance = 0.065f;
	public Vector2 screenSize = new Vector2 (4.88f, 3.05f);
	public Vector3 viewerPosition = new Vector3 (2.44f, 1.65f, 1.2f);

	public bool moveCameraToViewer = false;

	[NonSerialized][HideInInspector] public GameObject cameraRig;
	[NonSerialized][HideInInspector] public GameObject[] cameraGOs;
	[NonSerialized][HideInInspector] public Camera[] cameras;
	[NonSerialized][HideInInspector] public RenderTexture[] renderTextures;
    [NonSerialized] [HideInInspector] public GameObject debugRig;
    [NonSerialized] [HideInInspector] public GameObject debugPosRig;
    [NonSerialized][HideInInspector] public bool isInitialized = false;

	private Camera originalCam;
	private GameObject[] debugQuads;

	void Awake () {
		Debug.Log("Setting up Cave Camera Rig");
		originalCam = gameObject.GetComponent<Camera> ();
		if (moveCameraToViewer) {
			transform.localScale = Vector3.one;
			transform.localPosition = new Vector3(viewerPosition.x - screenSize.x / 2f, viewerPosition.y, viewerPosition.z - screenSize.y);
			transform.localRotation = Quaternion.identity;
		}
        if (stereoMode == CaveStereoOption.Auto)
        {
            if (Screen.currentResolution.width > renderTextureSize.x)
                stereoMode = CaveStereoOption.Stereo;
            else
                stereoMode = CaveStereoOption.Mono;
        }
        if (screens == CaveScreenOption.Auto)
        {
            if (Screen.currentResolution.height > renderTextureSize.y)
                screens = CaveScreenOption.FrontAndFloor;
            else
                screens = CaveScreenOption.Front;
        }
        SetupCameraRig();
        SetupDebugRig ();
        ConfigureOriginalCam ();
		isInitialized = true;
	}

	void ConfigureOriginalCam () {
		originalCam.cullingMask = (1 << CAVE_LAYER);
		originalCam.clearFlags = CameraClearFlags.SolidColor;
	}

	void SetupCameraRig () {
		cameraRig = new GameObject("Cave Camera Rig");
        cameraRig.transform.SetParent(transform);
        cameraRig.transform.localPosition = Vector3.zero;
        cameraRig.transform.localRotation = Quaternion.identity;

        cameraGOs = new GameObject[4];
		cameras = new Camera[4];
		if (renderToTexture)
			renderTextures = new RenderTexture[4];

        Type[] skipList = { typeof(Transform), typeof(CaveCameraRig), typeof(AudioListener), typeof(FlareLayer) };
		for (int n = 0; n < 4; n++) {
            cameraGOs[n] = eMPlus.ObjectCloner.Clone(gameObject, skipList);
            cameraGOs[n].name = "Cave Cam " + (n % 2 == 0? "Front" : "Floor") + " " + (n < 2 ? "Left" : "Right");
            cameraGOs[n].transform.SetParent(cameraRig.transform);
            cameraGOs[n].transform.localPosition = Vector3.zero;
			cameraGOs[n].transform.localRotation = Quaternion.identity;
			cameras [n] = cameraGOs [n].GetComponent<Camera> ();
			cameras [n].clearFlags = originalCam.clearFlags;
			cameras [n].backgroundColor = originalCam.backgroundColor;
			cameras [n].cullingMask = originalCam.cullingMask & 0x7fffffff;
			if (renderToTexture) {
				renderTextures [n] = new RenderTexture (renderTextureSize.x, renderTextureSize.y, 24);
				cameras [n].SetTargetBuffers(renderTextures[n].colorBuffer, renderTextures[n].depthBuffer);
			}
		}

        // make sure that the cameras are changed on first update
        _screens    = (CaveScreenOption)(((int)screens + 1) % 3);
        _stereoMode = (CaveStereoOption)(((int)stereoMode+ 1) % 3);
    }

    void SetupCamera(int cam, bool setupChanged = false)
    {
        if (setupChanged)
        {
            float w = originalCam.pixelRect.width;
            float h = originalCam.pixelRect.height;
            float dx = 0f;
            float dy = 0f;
            if (_stereoMode == CaveStereoOption.Stereo)
            {
                w /= 2f;
                dx = w;
            }
            if (_screens == CaveScreenOption.FrontAndFloor)
            {
                h /= 2f;
                dy = h;
            }
            bool active = false;
            switch ((CaveCamera)cam)
            {
                case CaveCamera.FrontLeft:
                    active = (_stereoMode != CaveStereoOption.Right) && (_screens == CaveScreenOption.FrontAndFloor || _screens == CaveScreenOption.Front);
                    cameraGOs[cam].transform.localEulerAngles = (new Vector3(0f, 0f, 0f));
                    dx = 0;
                    break;
                case CaveCamera.FloorLeft:
                    active = (_stereoMode != CaveStereoOption.Right) && (_screens == CaveScreenOption.FrontAndFloor || _screens == CaveScreenOption.Floor);
                    cameraGOs[cam].transform.localEulerAngles = (new Vector3(90f, 0f, 0f));
                    dx = dy = 0;
                    break;
                case CaveCamera.FrontRight:
                    active = (_stereoMode == CaveStereoOption.Stereo || _stereoMode == CaveStereoOption.Right) && (_screens == CaveScreenOption.FrontAndFloor || _screens == CaveScreenOption.Front);
                    cameraGOs[cam].transform.localEulerAngles = (new Vector3(0f, 0f, 0f));
                    break;
                case CaveCamera.FloorRight:
                    active = (_stereoMode == CaveStereoOption.Stereo || _stereoMode == CaveStereoOption.Right) && (_screens == CaveScreenOption.FrontAndFloor || _screens == CaveScreenOption.Floor);
                    cameraGOs[cam].transform.localEulerAngles = (new Vector3(90f, 0f, 0f));
                    dy = 0;
                    break;
            }
            Debug.Log((CaveCamera)cam + " (" + (active ? 1 : 0) + ") : " + dx + ", " + dy + ", " + w + ", " + h);

            if (renderToTexture)
                cameras[cam].pixelRect = new Rect(0f, 0f, renderTextureSize.x, renderTextureSize.y);
            else
                cameras[cam].pixelRect = new Rect(dx, dy, w, h);

            //only enable cameras that are being displayed
            cameraGOs[cam].SetActive(active);

            // make sure that the debug quads don't try to draw the left camera when it's not being rendered
            if (renderToTexture)
            {
                for (int n = 0; n < 2; n++)
                    debugQuads[n].GetComponent<MeshRenderer>().material.SetTexture("_MainTex", renderTextures[_stereoMode == CaveStereoOption.Right ? n + 2 : n]);
            }
        }

        bool left = cam < 2;
        bool front = cam % 2 == 0;

        float sx = interOccularDistance * (left ? -0.5f : 0.5f) * (_stereoMode == CaveStereoOption.Mono ? 0f : 1f);
        cameraGOs[cam].transform.localPosition = new Vector3(sx, 0, 0);

        cameras[cam].ResetProjectionMatrix();
        if (front)
            cameras[cam].fieldOfView = 2f * Mathf.Rad2Deg * Mathf.Atan2(screenSize.y * 0.5f, screenSize.y - viewerPosition.z);
        else
            cameras[cam].fieldOfView = 2f * Mathf.Rad2Deg * Mathf.Atan2(screenSize.y * 0.5f, viewerPosition.y);

        float shiftx = (sx + viewerPosition.x - screenSize.x * 0.5f) / screenSize.x;
        float shifty = ((front ? viewerPosition.y : viewerPosition.z) - screenSize.y * 0.5f) / screenSize.y;

        Matrix4x4 mat = cameras[cam].projectionMatrix;
        mat[0, 2] = -shiftx * 2f;
        mat[1, 2] = -shifty * 2f;
        cameras[cam].projectionMatrix = mat;
    }

    void SetupDebugRig()
    {
        debugRig = new GameObject("Cave Debug Rig");
        debugRig.transform.SetParent(transform);
        debugRig.transform.localPosition = Vector3.zero;
        debugRig.transform.localRotation = Quaternion.identity;

        debugPosRig = new GameObject("Cave Position Debug");
        debugPosRig.transform.SetParent(cameraRig.transform);
        debugPosRig.transform.localPosition = Vector3.zero;
        debugPosRig.transform.localRotation = Quaternion.identity;

        debugQuads = new GameObject[8];

        for (int n = 0; n < 2; n++)
        {
            debugQuads[n] = GameObject.CreatePrimitive(PrimitiveType.Quad);
            debugQuads[n].name = "Cave Screen Debug " + n;
            debugQuads[n].transform.SetParent(debugRig.transform);
            debugQuads[n].layer = CAVE_LAYER;
            switch (n)
            {
                case 0:
                    debugQuads[n].transform.localEulerAngles = (new Vector3(0f, 0f, 0f));
                    debugQuads[n].transform.localPosition = new Vector3(0f, 0f, 1f);
                    break;
                case 1:
                    debugQuads[n].transform.localEulerAngles = (new Vector3(90f, 0f, 0f));
                    debugQuads[n].transform.localPosition = new Vector3(0f, -1f, 0f);
                    break;
            }
            if (renderToTexture)
            {
                debugQuads[n].GetComponent<MeshRenderer>().material.SetTexture("_MainTex", renderTextures[n]);
                debugQuads[n].GetComponent<MeshRenderer>().material.shader = Shader.Find("Unlit/Texture");
            }
            else
            {
                debugQuads[n].GetComponent<MeshRenderer>().material.shader = Shader.Find("UI/Unlit/Transparent");
                debugQuads[n].GetComponent<MeshRenderer>().material.SetColor("_Color", new Color(1.0f - n * 0.5f, 0.5f + n * 0.5f, 0.5f, 0.5f));
            }
        }
        for (int n = 2; n < 8; n++)
        {
            debugQuads[n] = GameObject.CreatePrimitive(PrimitiveType.Quad);
            debugQuads[n].name = "Cave Position Debug " + (n < 4 ? "x" : (n > 5 ? "IOD" : (n == 4 ? "y" : "z"))) + " Position " + (n % 2);
            debugQuads[n].transform.SetParent(debugPosRig.transform);
//            debugQuads[n].layer = CAVE_LAYER;
            switch (n % 2)
            {
                case 0:
                    debugQuads[n].transform.localEulerAngles = (new Vector3(0f, 0f, 0f));
                    debugQuads[n].transform.localPosition = new Vector3(0f, 0f, 1f);
                    break;
                case 1:
                    debugQuads[n].transform.localEulerAngles = (new Vector3(90f, 0f, 0f));
                    debugQuads[n].transform.localPosition = new Vector3(0f, -1f, 0f);
                    break;
            }
            debugQuads[n].GetComponent<MeshRenderer>().material.shader = Shader.Find("UI/Unlit/Transparent");
            if (n < 6)
                debugQuads[n].GetComponent<MeshRenderer>().material.SetColor("_Color", new Color(1f, 0f, 1f, 0.75f));
            else
                debugQuads[n].GetComponent<MeshRenderer>().material.SetColor("_Color", new Color(0.25f, 0.25f, 0.25f, 0.5f));
        }
    }

    void Update()
	{
        // reconfigure cameras
        bool setupChanged = (screens != _screens || stereoMode != _stereoMode);
        if (setupChanged)
        {
            Debug.Log("new camera/screen setup: " + screens + " / " + stereoMode);
            _screens = screens;
            _stereoMode = stereoMode;
        }
        for (int n = 0; n < 4; n++)
            SetupCamera(n, setupChanged);

        if (debug) {
            debugRig.SetActive(true);
            debugPosRig.SetActive(true);

            // screen quads
            for (int n = 0; n < 2; n++)
            {
                if (_screens == CaveScreenOption.FrontAndFloor || (n ==  (int)_screens))
                    debugQuads[n].transform.localScale = new Vector3(screenSize.x, screenSize.y, 1f);
                else
                    debugQuads[n].transform.localScale = new Vector3(0f, 0f, 1f);
                debugQuads[n].transform.localPosition = (n == 0 ? new Vector3(0f, 0f, screenSize.y * 0.5f) : new Vector3(0f, -screenSize.y * 0.5f, 0f));
                debugQuads[n].transform.localPosition += new Vector3(screenSize.x * 0.5f, screenSize.y * 0.5f, screenSize.y * 0.5f) - viewerPosition;
            }

            // position markers
            float w = 0.01f;
            float iod = _stereoMode == CaveStereoOption.Mono ? w * 0.5f : interOccularDistance;
            debugQuads[2].transform.localScale = new Vector3(w, screenSize.y, 1f);
            debugQuads[3].transform.localScale = new Vector3(w, screenSize.y, 1f);
            debugQuads[4].transform.localScale = new Vector3(screenSize.x, w, 1f);
            debugQuads[5].transform.localScale = new Vector3(screenSize.x, w, 1f);
            debugQuads[6].transform.localScale = new Vector3(iod, screenSize.y, 1f);
            debugQuads[7].transform.localScale = new Vector3(iod, screenSize.y, 1f);

            debugQuads[2].transform.localPosition = new Vector3(0f, screenSize.y * 0.5f - viewerPosition.y, screenSize.y - viewerPosition.z);
            debugQuads[3].transform.localPosition = new Vector3(0f, screenSize.y * 0.0f - viewerPosition.y, screenSize.y * 0.5f - viewerPosition.z);
            debugQuads[4].transform.localPosition = new Vector3(screenSize.x * 0.5f - viewerPosition.x, 0f, screenSize.y - viewerPosition.z);
            debugQuads[5].transform.localPosition = new Vector3(screenSize.x * 0.5f - viewerPosition.x, screenSize.y * 0.0f - viewerPosition.y, 0f);
            debugQuads[6].transform.localPosition = new Vector3(0f, screenSize.y * 0.5f - viewerPosition.y, screenSize.y - viewerPosition.z);
            debugQuads[7].transform.localPosition = new Vector3(0f, screenSize.y * 0.0f - viewerPosition.y, screenSize.y * 0.5f - viewerPosition.z);
        }
        else
        {
            debugRig.SetActive(false);
            debugPosRig.SetActive(false);
        }
    }
}